@@ -428,7 +428,6 @@ MetaR <-  function(meta, cor = .50) {
   # Args:
   #   meta: data.frame with r (correlation coefficients) and n (sample size)for 
   #   each study.
-  # Returns:
   #   Aggregated effect size (one row per study) based on Hunter and Schmidt's (2004)
   #   approach to aggregation of dependent r's (see chapter 10,  pp. 435-8).
   #   Adds study weights based of a Fisher's z transformation of r's (see chapter 14, 
@@ -443,7 +442,7 @@ MetaR <-  function(meta, cor = .50) {
   meta$l.ci95 <- (exp(2*meta$l.ci95z)-1)/(exp(2*meta$l.ci95z) + 1)
   meta$u.ci95 <- (exp(2*meta$u.ci95z)-1)/(exp(2*meta$u.ci95z) + 1)  
   meta$z.score <- meta$z/sqrt(meta$var.z)
-  meta$p.value <- 2*(1-pt(abs(meta$z.score),  meta$n-1))
+  meta$p.value <- 2*pnorm(abs(meta$z.score), lower.tail=FALSE)
   meta$wi <-  1/meta$var.z  # computing weight for each study
   meta$wiTi <- meta$wi*meta$z  # used to calculate omnibus
   meta$wiTi2 <- meta$wi*(meta$z)^2  # used to calculate omnibus
@@ -494,7 +493,7 @@ OmnibusES<-  function(meta,  var="weighted" ) {
   var.Tz.agg <- 1/sum.wi  # omnibus var.z
   se.Tz.agg <- sqrt(var.Tz.agg) 
   z.value  <-  Tz.agg/se.Tz.agg
-  p.value <-  round(2*(1-pt(abs(z.value),  df)), 6)  
+  p.value <-  2*pnorm(abs(z.value), lower.tail=FALSE)
   T.agg <- (exp(2*Tz.agg)-1)/(exp(2*Tz.agg) + 1)  # fixed effect (FE) omnibus effect size (r)
   var.T.agg <- (exp(2*var.Tz.agg)-1)/(exp(2*var.Tz.agg) + 1)  # FE omnibus var.r
   lower.ci.Tz <- Tz.agg-1.96*se.Tz.agg
@@ -517,7 +516,7 @@ OmnibusES<-  function(meta,  var="weighted" ) {
     var.Tz.agg.tau <-  1/sum.wi.tau 
     se.Tz.agg.tau <- sqrt(var.Tz.agg.tau)
     z.valueR  <-  Tz.agg.tau/se.Tz.agg.tau
-    p.valueR <-  round(2*(1-pt(abs(z.valueR),  df)), 6)
+    p.valueR <-  2*pnorm(abs(z.valueR), lower.tail=FALSE)
     T.agg.tau <- (exp(2*Tz.agg.tau)-1)/(exp(2*Tz.agg.tau) + 1)
     var.T.agg.tau <- (exp(2*var.Tz.agg.tau)-1)/(exp(2*var.Tz.agg.tau) + 1)
     lower.ci.Tz.tau <- Tz.agg.tau-1.96*se.Tz.agg.tau
@@ -532,7 +531,7 @@ OmnibusES<-  function(meta,  var="weighted" ) {
     var.Tz.agg.tau <-  ifelse(unwgtvar.Tz.agg.tau <= 0, 0, unwgtvar.Tz.agg.tau)  #if var < 0,  its set to 0
     se.Tz.agg.tau <- sqrt(var.Tz.agg.tau)
     z.valueR  <-  Tz.agg.tau/se.Tz.agg.tau
-    p.valueR <-  round(2*(1-pt(abs(z.valueR),  df)), 6)
+    p.valueR <-  2*pnorm(abs(z.valueR), lower.tail=FALSE)
     T.agg.tau <- (exp(2*Tz.agg.tau)-1)/(exp(2*Tz.agg.tau) + 1)
     var.T.agg.tau <- (exp(2*var.Tz.agg.tau)-1)/(exp(2*var.Tz.agg.tau) + 1)
     lower.ci.Tz.tau <- Tz.agg.tau-1.96*se.Tz.agg.tau
@@ -561,156 +560,6 @@ OmnibusES<-  function(meta,  var="weighted" ) {
 # Now,  if there is significant heterogeneity (p_homog < .05),  look for moderators.
 
 ##================= Categorical Moderator Analysis ================##
-# requires plyr package
-
-# Fixed effects intermediate level functions:
-
-f1 <- function(meta) c(k=length(meta$k), sum.wi=sum(meta$wi, na.rm=TRUE), 
-                       sum.wiTi=sum(meta$wiTi, na.rm=TRUE), sum.wiTi2=sum(meta$wiTi2,
-                       na.rm=TRUE))
-f2 <- function(modsum) c(K=modsum$k, ES=modsum$sum.wiTi/modsum$sum.wi, 
-                         Var=1/modsum$sum.wi, 
-                         Q=modsum$sum.wiTi2-(modsum$sum.wiTi^2)/modsum$sum.wi)
-
-# pick_one added 2.17.10 (thanks to Hadley Wickham for assistance)
-
-pick_one <- function(x) {
- if (length(x) == 1) return(x)
- sample(x, 1)
-}
-
-# Intermediate level function: sums relevant es information by each moderator level 
-cat_sum1 <- function(meta, mod) {
-  m <- meta
-  m$mod <- mod
-  compl<-!is.na(m$mod)
-  meta<-m[compl,]
-  #meta <- agg_r2(m,m$mod)
-  #meta <- ddply(m,  c("id", "mod"),  summarize,  r = aggrs(r), n=mean(n))
-  #meta <- do.call(rbind, lapply(split(meta, meta$id), 
-  #          function(.data) .data[sample(nrow(.data), 1),]))
-  meta$z  <- 0.5*log((1 + meta$r)/(1-meta$r))   
-  meta$var.z <- 1/(meta$n-3)
-  meta$wi <-  1/meta$var.z
-  meta$wiTi <- meta$wi*meta$z
-  meta$wiTi2 <- meta$wi*(meta$z)^2
-  meta$k <- length(meta$mod)
-  mod <- meta$mod
-  sum.mod <- by(meta, mod, f1)
-  sums <- cbind(expand.grid(dimnames(sum.mod)), do.call(rbind, sum.mod))
-  return(sums)
-}
-# Intermediate level function: adds an overall category to moderators
-
-cat_sum2 <-  function(meta, mod) {
-  catsum <- cat_sum1(meta, mod)
-  wi <- sum(catsum$sum.wi)
-  wiTi <- sum(catsum$sum.wiTi)
-  wiTi2 <- sum(catsum$sum.wiTi2)
-  k <- sum(catsum$k)
-  mod <- catsum$mod
-  catsums <- rbind(catsum, data.frame(mod="Overall", sum.wi=wi, sum.wiTi=wiTi, 
-                   sum.wiTi2=wiTi2, k=k))
-  return(catsums)
-}
-
-# Intermediate level function: derives mean ES & Var for each group
-
-mod_mean <- function(meta,  mod) {
-  modsum <- cat_sum2(meta,  mod)
-  sum.values <- by(modsum, modsum$mod, f2)
-  sums <- cbind(expand.grid(dimnames(sum.values)), do.call(rbind, sum.values))
-  colnames(sums) <- c("Mod", "K", "ES", "Var", "Q")
-  return(sums)
-}
-
-# Intermediate level function: adds 95% confidence intervals
-mod_sig.z <- function(meta,  mod) {
-  mod.es <- mod_mean(meta, mod)
-  mod.es$L.95ci <- mod.es$ES-1.96*sqrt(mod.es$Var)
-  mod.es$U.95ci <- mod.es$ES + 1.96*sqrt(mod.es$Var)
-  return(mod.es)}
-
-# Random effects intermediate level functions:
-
-f3 <- function(meta) c(k=length(meta$k), sum.wi.tau=sum(meta$wi.tau, 
-                       na.rm=TRUE),  sum.wiTi.tau=sum(meta$wiTi.tau, na.rm=TRUE), 
-                       sum.wiTi2.tau=sum(meta$wiTi2.tau, na.rm=TRUE))
-f4 <- function(modsum) c(K=modsum$k, ES=modsum$sum.wiTi.tau/modsum$sum.wi.tau, 
-                         Var=1/modsum$sum.wi.tau,  
-                         Q=modsum$sum.wiTi2.tau-(modsum$sum.wiTi.tau^2)/modsum$sum.wi.tau)
-
-# Intermediate level function: sums relevant es information by each moderator level 
-
-cat_sum1R <- function(meta, mod) {
-  m <- meta
-  m$mod <- mod
-  compl<-!is.na(m$mod)
-  meta<-m[compl,]
-  #meta <- agg_r2(m,m$mod)
-  #meta <- ddply(m,  c("id", "mod"),  summarize,  r = aggrs(r), n=mean(n))
-  #meta <- do.call(rbind, lapply(split(meta, meta$id), 
-  #          function(.data) .data[sample(nrow(.data), 1),]))
-  meta$z <- 0.5*log((1 + meta$r)/(1-meta$r))   
-  meta$var.z <- 1/(meta$n-3)
-  meta$wi <- 1/meta$var.z
-  meta$wi2 <- (1/meta$var.z)^2
-  meta$wiTi <- meta$wi*meta$z
-  meta$wiTi2 <- meta$wi*(meta$z)^2
-  meta$k <- meta$mod
-  mod <- meta$mod
-  sum.wi <- sum(meta$wi, na.rm=TRUE)    
-  sum.wi2 <- sum(meta$wi2, na.rm=TRUE)
-  sum.wiTi <- sum(meta$wiTi, na.rm=TRUE)    
-  sum.wiTi2 <- sum(meta$wiTi2, na.rm=TRUE)
-  comp <- sum.wi-sum.wi2/sum.wi
-  Q <- sum.wiTi2-(sum.wiTi^2)/sum.wi                         
-  k <- sum(!is.na(meta$r))                                  
-  df <- k-1      
-  tau <- (Q-(k - 1))/comp  #random effects variance
-  meta$var.tau <- meta$var.z + tau
-  meta$wi.tau <- 1/meta$var.tau
-  meta$wiTi.tau <- meta$wi.tau*meta$z
-  meta$wiTi2.tau <- meta$wi.tau*(meta$z)^2
-  sum.mod <- by(meta, mod, f3)
-  sums <- cbind(expand.grid(dimnames(sum.mod)), do.call(rbind, sum.mod))
-  names(sums)[1] <- "mod"
-  return(sums)
-}
-
-# Intermediate level function: adds an overall category to moderators
-
-cat_sum2R <- function(meta, mod) {
-  catsum <- cat_sum1R(meta, mod)
-  wi.tau <- sum(catsum$sum.wi.tau)
-  wiTi.tau <- sum(catsum$sum.wiTi.tau)
-  wiTi2.tau <- sum(catsum$sum.wiTi2.tau)
-  k <- sum(catsum$k)
-  mod <- meta$mod
-  catsums <- rbind(catsum, data.frame(mod="Overall", sum.wi.tau=wi.tau, 
-  sum.wiTi.tau=wiTi.tau,  sum.wiTi2.tau=wiTi2.tau, k=k))
-  return(catsums)
-}
-
-
-# Intermediate level function: derives mean ES & Var for each group
-
-mod_meanR <- function(meta,  mod) {
-  modsum <- cat_sum2R(meta,  mod)
-  sum.values <- by(modsum, modsum$mod, f4)
-  sums <- cbind(expand.grid(dimnames(sum.values)), do.call(rbind, sum.values))
-  colnames(sums) <- c("Mod", "K", "ES", "Var", "Q")
-  return(sums)
-}
-
-# Intermediate level function: adds 95% confidence intervals
-
-mod_sig.zR <- function(meta,  mod) {
-  mod.es <- mod_meanR(meta, mod)
-  mod.es$L.95ci <- mod.es$ES-1.96*sqrt(mod.es$Var)
-  mod.es$U.95ci <- mod.es$ES + 1.96*sqrt(mod.es$Var)
-  return(mod.es)}
-
 # Single predictor categorical moderator function (fixed effects)
 
 CatModf <-  function(meta,  mod) {
@@ -722,26 +571,47 @@ CatModf <-  function(meta,  mod) {
   # Returns:
   #   Fixed effects moderator means per group, k per group, 95% confidence intervals,
   #   z-value, p-value, variances, standard errors, Q, df(Q), and I-squared.
-  mod.sig <- mod_sig.z(meta, mod)
-  mod.sig$ES <- (exp(2*mod.sig$ES)-1)/(exp(2*mod.sig$ES) + 1)
-  mod.sig$Var <- (exp(2*mod.sig$Var)-1)/(exp(2*mod.sig$Var) + 1)
-  mod.sig$Std.Err <- sqrt(mod.sig$Var)
-  k <- mod.sig[mod.sig$Mod=="Overall", "K"]  
-  df <- k-1
-  mod.sig$z.value <- mod.sig$ES/mod.sig$Std.Err
-  mod.sig$p.value <- round(2*(1-pt(abs(mod.sig$z.value),  df)), 6)
-  mod.sig$L.95ci <- (exp(2*mod.sig$L.95ci)-1)/(exp(2*mod.sig$L.95ci) + 1)
-  mod.sig$U.95ci <- (exp(2*mod.sig$U.95ci)-1)/(exp(2*mod.sig$U.95ci) + 1)
-  mod.sig$df.Q <- mod.sig$K-1
-  mod.sig$p_homog <- ifelse(mod.sig$df.Q==0,  1,  round(pchisq(mod.sig$Q, mod.sig$df.Q, lower=FALSE), 5)) 
-  mod.sig$I2 <- (mod.sig$Q-(mod.sig$K-1))/mod.sig$Q   #I-squared  
-  mod.sig$I2 <- ifelse(mod.sig$I2<0, 0, mod.sig$I2)     
-  mod.sig$I2 <- paste(round(mod.sig$I2*100, 4),  "%",  sep="")
-  mod.sig2 <- mod.sig[, c(1, 2, 3, 6, 7, 9, 10, 4, 8, 5, 11, 12, 13)] #effects due to heterogeneity rather than chance (p263)
-  rownames(mod.sig2) <- NULL
-  colnames(mod.sig2) <- c("Mod", "k", "ES", "CI.lower", "CI.upper",  "Z", 
-  "p",  "var.ES", "SE",  "Q", "df", "p.h",  "I2")
-  return(mod.sig2)
+  meta$mod <- as.character(mod)
+  meta$z  <- 0.5*log((1 + meta$r)/(1-meta$r))   
+  meta$var.z <- 1/(meta$n-3)
+  meta$k <- 1
+  meta$TW <- 1/meta$var.z
+  meta$TWD <- meta$TW*meta$z
+  meta$TWDS <- meta$TWD*meta$z
+  out <- aggregate(meta[c("k","TW","TWD", "TWDS")], by=meta["mod"], FUN=sum)
+  out$mod <- as.character(out$mod)
+  lastrow <- dim(out)[1] + 1
+  out[lastrow, 2:5] <- apply(out[,-1], 2, FUN=sum)
+  out$mod[lastrow] <- "Overall"
+  out$z <- out$TWD/out$TW
+  out$var.z <- 1/out$TW
+  out$se.z <- sqrt(out$var.z)
+  out$Q <- out$TWDS - out$TWD^2/out$TW
+  out$df <- out$k - 1
+  out$z.value <- out$z/out$se.z
+  out$p.value <- 2*pnorm(abs(out$z.value), lower.tail=FALSE)
+  out$L.95ci <- out$z-1.96*out$se.z
+  out$U.95ci  <- out$z+1.96*out$se.z
+  out$p_homog <- ifelse(out$df==0,  1,  pchisq(out$Q, out$df, lower=FALSE)) 
+  out$I2 <- (out$Q-(out$df))/out$Q   #I-squared  
+  out$I2 <- ifelse(out$I2<0, 0, out$I2)     
+  out$I2 <- paste(round(out$I2*100, 4),  "%",  sep="")
+  # convert back to r
+  out$r <- (exp(2*out$z)-1)/(exp(2*out$z) + 1)
+  out$var.r <- (exp(2*out$var.z)-1)/(exp(2*out$var.z) + 1)
+  out$se.r <- sqrt(out$var.r)
+  out$L.95ci <- (exp(2*out$L.95ci)-1)/(exp(2*out$L.95ci) + 1)
+  out$U.95ci <- (exp(2*out$U.95ci)-1)/(exp(2*out$U.95ci) + 1)
+  out$TW <- NULL
+  out$TWD <- NULL
+  out$TWDS <- NULL
+  out$z <- NULL
+  out$var.z <- NULL
+  out$se.z <- NULL
+  colnames(out) <- c("Mod", "k", "Q", "df", "Z", "p", "CI.lower",
+                    "CI.upper", "p.h", "I2","ES", "Var", "SE")
+  out <- out[, c(1, 2, 11, 12, 13, 7, 8,5, 6, 3, 4, 9, 10 )] 
+  return(out)
 }
 
 # Fixed effect single predictor categorical moderator Q-statistic
@@ -787,7 +657,8 @@ CatCompf <- function(meta,  mod, x1, x2,  method= "post.hoc1") {
   # Returns:
   #   Random effects moderator means per group, mean difference (d), variance of difference,
   #   p-value, and 95% confidence intervals. 
-  modsig <- mod_sig.z(meta, mod)
+  modsig <- CatModf(meta, mod)
+  modsig$Mod <- as.factor(modsig$Mod)
   com1 <- levels(modsig$Mod)[x1]  # first level of moderator
   com2 <- levels(modsig$Mod)[x2]  # second level of moderator
   x1.es <- modsig[modsig$Mod==com1, "ES"]
@@ -802,10 +673,6 @@ CatCompf <- function(meta,  mod, x1, x2,  method= "post.hoc1") {
   m$mod <- mod
   compl<-!is.na(m$mod)
   meta<-m[compl,]
-  #meta <- agg_r2(m,m$mod)
-  #meta <- ddply(m,  c("id", "mod"),  summarize,  r = aggrs(r), n=mean(n))
-  #meta <- do.call(rbind, lapply(split(meta, meta$id), 
-  #        function(.data) .data[sample(nrow(.data), 1),]))
   meta$z <- 0.5*log((1 + meta$r)/(1-meta$r))   
   meta$var.z <- 1/(meta$n-3)
   meta$wi <-  1/meta$var.z
@@ -838,66 +705,76 @@ CatCompf <- function(meta,  mod, x1, x2,  method= "post.hoc1") {
   return(fit) 
 }
 
-# Function for planned comparisons between 2 levels of moderator (random effects)
-
 CatModr <-  function(meta,  mod) {
-  # Computes single predictor random effects categorical moderator analysis. 
-  # Computations derived from chapter 15, Cooper et al. (2009). 
+  # Computes single predictor categorical moderator analysis. Computations derived from 
+  # chapter 15, Cooper et al. (2009). 
   # Args:
-  #   meta: data.frame with r (correlation coefficients) and n (sample size)for each study.
+  #   meta: data.frame with g (standardized mean diff),var.g (variance of g),
+  #         n.1 (grp 1 sample size), n.2 (grp 2 sample size).
   #   mod: Categorical moderator variable used for moderator analysis.    
   # Returns:
-  #   Random effects moderator means per group, k per group, 95% confidence intervals,
+  #   Fixed effects moderator means per group, k per group, 95% confidence intervals,
   #   z-value, p-value, variances, standard errors, Q, df(Q), and I-squared.
-  mod.sig <- mod_sig.zR(meta, mod)
-  mod.sig$ES <- (exp(2*mod.sig$ES)-1)/(exp(2*mod.sig$ES) + 1)
-  mod.sig$Var <-(exp(2*mod.sig$Var)-1)/(exp(2*mod.sig$Var) + 1)
-  mod.sig$Std.Err <- sqrt(mod.sig$Var)
-  k <- sum(!is.na(mod)) 
-  df <-  k-1
-  mod.sig$z.value <- mod.sig$ES/mod.sig$Std.Err
-  mod.sig$p.value <- round(2*(1-pt(abs(mod.sig$z.value),  df)), 6)
-  mod.sig$L.95ci <- (exp(2*mod.sig$L.95ci)-1)/(exp(2*mod.sig$L.95ci) + 1)
-  mod.sig$U.95ci <-  (exp(2*mod.sig$U.95ci)-1)/(exp(2*mod.sig$U.95ci) + 1)
-  mod.sig$Q <- mod.sig$Q
-  mod.sig$df.Q <- mod.sig$K-1
-  mod.sig$p_homog <- ifelse(mod.sig$df.Q==0, 1, round(pchisq(mod.sig$Q, mod.sig$df.Q, lower=FALSE), 5)) 
-  mod.sig$I2 <- (mod.sig$Q-(mod.sig$K-1))/mod.sig$Q   #I-squared  
-  mod.sig$I2 <- ifelse(mod.sig$I2<0, 0, mod.sig$I2)     
-  mod.sig$I2 <- paste(round(mod.sig$I2*100, 4),  "%",  sep="")
-  mod.sig2 <- mod.sig[, c(1, 2, 3, 6, 7, 9, 10, 4, 8, 5, 11, 12, 13)]
-  rownames(mod.sig2) <- NULL
-  colnames(mod.sig2) <- c("Mod", "k", "ES", "CI.lower", "CI.upper",  "Z", 
-  "p",  "var.ES", "SE",  "Q", "df", "p.h",  "I2")
-  return(mod.sig2)
-}  
-
-# Intermediate function for random effects Q-statistic test
-
-qrs <- function(meta,  mod) {
-  mod.sig <- mod_sig.zR(meta, mod)
-  mod.sig$ES <- (exp(2*mod.sig$ES)-1)/(exp(2*mod.sig$ES) + 1)
-  mod.sig$Var <- (exp(2*mod.sig$Var)-1)/(exp(2*mod.sig$Var) + 1)
-  mod.sig$Std.Err <-  sqrt(mod.sig$Var)
-  k <-  mod.sig[mod.sig$Mod=="Overall", "K"] #sum(!is.na(mod)) 
-  df <-  k-1
-  mod.sig$z.value <- mod.sig$ES/mod.sig$Std.Err
-  mod.sig$p.value <- round(2*(1-pt(abs(mod.sig$z.value),  df)), 6)
-  mod.sig$L.95ci <- (exp(2*mod.sig$L.95ci)-1)/(exp(2*mod.sig$L.95ci) + 1)
-  mod.sig$U.95ci <- (exp(2*mod.sig$U.95ci)-1)/(exp(2*mod.sig$U.95ci) + 1)
-  mod.sig$df.Q <- mod.sig$K-1
-  mod.sig$p_homog <-  ifelse(mod.sig$df.Q==0,  1,  round(pchisq(mod.sig$Q, mod.sig$df.Q, lower=FALSE), 5)) #work on this:if(mod.sig$df.Q[mod.sig$df.Q==0]) 1.0000 else pchisq(mod.sig$Q, mod.sig$df.Q, lower=FALSE)
-  mod.sig$I2 <- (mod.sig$Q-(mod.sig$K-1))/mod.sig$Q      #I-squared is prop of total 
-  mod.sig$I2 <- ifelse(mod.sig$I2<0, 0, mod.sig$I2)        #variation in the est of tmt
-  #mod.sig$I2 <- ifelse(mod.sig$p_homog==1, mod.sig$I2==0,mod.sig$I2)
-  mod.sig$I2 <- paste(round(mod.sig$I2*100, 4),  "%",  sep="")
-  mod.sig2 <- mod.sig[, c(1, 2, 3, 6, 7, 9, 10, 4, 8, 5, 11, 12, 13)] #effects due to heterogeneity rather than chance (p263)
-  rownames(mod.sig2) <- NULL
-  colnames(mod.sig2) <- c("Mod", "k", "ES", "CI.lower", "CI.upper",  "Z", 
-  "p",  "var.ES", "SE",  "Q", "df", "p.h",  "I2")
-  return(mod.sig2)
+  meta$mod <- as.character(mod)
+  meta$z  <- 0.5*log((1 + meta$r)/(1-meta$r))   
+  meta$var.z <- 1/(meta$n-3)
+  meta$k <- 1
+  meta$TW <- 1/meta$var.z
+  meta$TWS <- meta$TW^2
+  meta$TWD <- meta$TW*meta$z
+  meta$TWDS <- meta$TWD*meta$z
+  out <- aggregate(meta[c("k","TW","TWS", "TWD", "TWDS")], by=meta["mod"], FUN=sum)
+  out$Q <- out$TWDS - out$TWD^2/out$TW
+  Q <- sum(out$Q)
+  out$df <- out$k - 1
+  df <- sum(out$df)
+  out$c <- out$TW - (out$TWS/out$TW) # 19.37 ch. 19 Borenstein (2009)
+  c <- sum(out$c)
+  TSw <- (Q - df)/c  # 19.38 ch. 19 Borenstein (2009)  
+  tau <- ifelse(TSw < 0, 0, TSw)
+  # add the tau variance to each indiv study & then compute TW again
+  meta$var.tau <- meta$var.z + tau
+  meta$TW.tau <- 1/meta$var.tau
+  meta$TWD.tau <- meta$TW.tau*meta$z
+  meta$TWDS.tau <- meta$TWD.tau*meta$z
+  out <- aggregate(meta[c("k","TW.tau","TWD.tau", 
+                   "TWDS.tau")], by=meta["mod"], FUN=sum)
+  lastrow <- dim(out)[1] + 1
+  out[lastrow, 2:5] <- apply(out[,-1], 2, FUN=sum)
+  out$mod[lastrow] <- "Overall"
+  out$z <- out$TWD.tau/out$TW.tau
+  out$var.z <- 1/out$TW.tau
+  out$z <- out$TWD.tau/out$TW.tau
+  out$se.z <- sqrt(out$var.z)
+  out$Q <- out$TWDS.tau - out$TWD.tau^2/out$TW.tau
+  out$df <- out$k - 1
+  out$z.value <- out$z/out$se.z
+  out$p.value <- 2*pnorm(abs(out$z.value), lower.tail=FALSE)
+  out$L.95ci <- out$z-1.96*out$se.z
+  out$U.95ci  <- out$z+1.96*out$se.z
+  out$p_homog <- ifelse(out$df==0,  1,  pchisq(out$Q, out$df, lower=FALSE)) 
+  out$I2 <- (out$Q-(out$df))/out$Q   # these values are not to be used 
+  out$I2 <- ifelse(out$I2<0, 0, out$I2)     
+  out$I2 <- paste(round(out$I2*100, 4),  "%",  sep="")
+  out$TW.tau <- NULL
+  out$TWD.tau <- NULL
+  out$TWDS.tau <- NULL
+  # convert back to r
+  out$r <- (exp(2*out$z)-1)/(exp(2*out$z) + 1)
+  out$var.r <- (exp(2*out$var.z)-1)/(exp(2*out$var.z) + 1)
+  out$se.r <- sqrt(out$var.r)
+  out$L.95ci <- (exp(2*out$L.95ci)-1)/(exp(2*out$L.95ci) + 1)
+  out$U.95ci <- (exp(2*out$U.95ci)-1)/(exp(2*out$U.95ci) + 1)
+  out$z <- NULL
+  out$var.z <- NULL
+  out$se.z <- NULL
+  colnames(out) <- c("Mod", "k", "Q", "df", "Z", "p", "CI.lower",
+                    "CI.upper", "p.h", "I2","ES", "Var", "SE")
+  out <- out[, c(1, 2, 11, 12, 13, 7, 8,5, 6, 3, 4, 9, 10 )] 
+  return(out)
 }
 
+
 #Q-statistic function (random effects)
 
 CatModrQ <-  function(meta,  mod) {
@@ -909,7 +786,7 @@ CatModrQ <-  function(meta,  mod) {
   # Returns:
   #   Random effects moderator Q-statistic, Q-within & between, df(Qw & Qb), and 
   #   homogeneity p-value within & between levels.
-  mod.sig <- qrs(meta, mod)
+  mod.sig <- CatModr(meta, mod)
   k <- mod.sig$k[mod.sig$Mod=="Overall"]                         #number of studies
   levels <- length(mod.sig$Mod)-1
   Qb.df <- levels-1 
@@ -957,7 +834,8 @@ CatCompr <- function(meta,  mod, x1, x2,  method= "post.hoc1") {
   # Returns:
   #   Random effects moderator means per group, mean difference (d), variance of difference,
   #   p-value, and 95% confidence intervals. 
-  modsig <- mod_sig.zR(meta, mod)
+  modsig <- CatModr(meta, mod)
+  modsig$Mod <- as.factor(modsig$Mod)
   com1 <- levels(modsig$Mod)[x1]  # first level of moderator
   com2 <- levels(modsig$Mod)[x2]  # second level of moderator
   x1.es <- modsig[modsig$Mod==com1, "ES"]
@@ -972,10 +850,6 @@ CatCompr <- function(meta,  mod, x1, x2,  method= "post.hoc1") {
   m$mod <- mod
   compl<-!is.na(m$mod)
   meta<-m[compl,]
-  #meta <- agg_r2(m,m$mod)
-  #meta <- ddply(m,  c("id", "mod"),  summarize,  r = aggrs(r), n=mean(n))
-  #meta <- do.call(rbind, lapply(split(meta, meta$id), 
-  #        function(.data) .data[sample(nrow(.data), 1),]))
   meta$z  <-  0.5*log((1 + meta$r)/(1-meta$r))   
   meta$var.z <- 1/(meta$n-3)
   meta$wi <-  1/meta$var.z
@@ -1109,10 +983,10 @@ MAreg1 <- function(meta, mod, method="random") {  # Single predictor meta-regres
     Bs.z <- summary(reg)$coef[, 1]
     Bs <-  (exp(2*Bs.z)-1)/(exp(2*Bs.z) + 1)
     newSE <- (exp(2*newSE.z)-1)/(exp(2*newSE.z) + 1)
-    t.adj <- round(Bs/newSE, 6)
-    p.adj <- round(2*(1-pt(abs(t.adj),  df)), 6)
-    lower.ci <- round(Bs-(t.crit*newSE), 6)  # 95% CI
-    upper.ci <- round(Bs + (t.crit*newSE), 6)  # 95% CI
+    t.adj <- Bs/newSE
+    p.adj <- 2*pnorm(abs(t.adj), lower.tail=FALSE)
+    lower.ci <- Bs-(t.crit*newSE)  # 95% CI
+    upper.ci <- Bs + (t.crit*newSE)  # 95% CI
     modelfit <- MRfit(reg0,reg)   # internal function to assess fit
     #sig <- symnum(p.adj,  corr = FALSE,  na = FALSE,  
     #              cutpoints <- c(0,  0.001,  0.01,  0.05,  0.1,  1), 
@@ -1129,10 +1003,10 @@ MAreg1 <- function(meta, mod, method="random") {  # Single predictor meta-regres
     Bs.z  <- summary(reg)$coef[, 1]  # Need an option to keep in z' 
     Bs <- (exp(2*Bs.z)-1)/(exp(2*Bs.z) + 1)
     newSE <- (exp(2*newSE.z)-1)/(exp(2*newSE.z) + 1)
-    t.adj <- round(Bs/newSE, 6)
-    p.adj <- round(2*(1-pt(abs(t.adj),  df)), 6)
-    lower.ci <- round(Bs-(t.crit*newSE), 6)  # 95% CI
-    upper.ci <- round(Bs + (t.crit*newSE), 6)  # 95% CI
+    t.adj <- Bs/newSE
+    p.adj <- 2*pnorm(abs(t.adj), lower.tail=FALSE)
+    lower.ci <- Bs-(t.crit*newSE)  # 95% CI
+    upper.ci <- Bs + (t.crit*newSE)  # 95% CI
     modelfit <- MRfit(reg0,reg)
     #sig <- symnum(p.adj,  corr = FALSE,  na = FALSE,  
     #              cutpoints <- c(0,  0.001,  0.01,  0.05,  0.1,  1), 
@@ -1165,10 +1039,10 @@ MAreg2  <-  function(reg) {  # Multivariate meta-regression
   Bs.z  <-  summary(reg)$coef[, 1]
   Bs <-  (exp(2*Bs.z)-1)/(exp(2*Bs.z) + 1)
   newSE <-  (exp(2*newSE.z)-1)/(exp(2*newSE.z) + 1)
-  t.adj  <-  round(Bs/newSE, 6)
-  p.adj  <-  round(2*(1-pt(abs(t.adj),  df)), 6)
-  lower.ci <- round(Bs-(t.crit*newSE), 6)     #95% CI
-  upper.ci <- round(Bs + (t.crit*newSE), 6)     #95% CI
+  t.adj  <-  Bs/newSE
+  p.adj  <-  2*pnorm(abs(t.adj), lower.tail=FALSE)
+  lower.ci <- Bs-(t.crit*newSE)     #95% CI
+  upper.ci <- Bs + (t.crit*newSE)     #95% CI
   #sig <- symnum(p.adj,  corr = FALSE,  na = FALSE,  
   #           cutpoints <- c(0,  0.001,  0.01,  0.05,  0.1,  1), 
   #           symbols <- c("***",  "**",  "*",  ".",  " ")) 
@@ -1375,7 +1249,7 @@ ForestPlot <- function(meta, method="random", title=NULL) {
      forest <- ggplot(meta,  aes(y = id, x=r))+    #aes(y = factor(id, levels=rev(levels(id))),  x = r))  +  
                     geom_vline(xintercept=0) + 
                     geom_point(data=omnibus, colour="red", size=8, shape=23) + 
-                    geom_point(aes(size=wi)) + 
+                    #geom_point(aes(size=wi)) + 
                     opts(title=title,  legend.position="none") + 
                     geom_errorbarh(aes(xmin = l.ci95,  xmax=u.ci95), size=.3, alpha=.6) + 
                     geom_vline(colour="red", linetype=2,  xintercept=T.agg) + 
@@ -1410,7 +1284,7 @@ ForestPlot <- function(meta, method="random", title=NULL) {
     forest <- ggplot(meta,  aes(y = id,x = r))+    #aes(y = factor(id, levels=rev(levels(id))),  x = r))  +  
                   geom_vline(xintercept=0) + 
                   geom_point(data=omnibus.tau, colour="red",  size=8,  shape=23) + 
-                  geom_point(aes(size=wi.tau)) + 
+                  #geom_point(aes(size=wi.tau)) + 
                   opts(title=title,  legend.position="none") + 
                   geom_errorbarh(aes(xmin = l.ci95,  xmax=u.ci95), size=.3, alpha=.6) + 
                   geom_vline(colour="red", linetype=2,  xintercept=T.agg.tau) + 
@@ -1456,7 +1330,7 @@ FunnelPlot <- function(meta, method="random",  title=NULL) {
     funnel <- ggplot(meta,  aes(y = se.z,  x = z))  +  
                     geom_vline(colour="black", linetype=1, 
                     xintercept=omnibus.z) + 
-                    geom_point(aes(size=wi)) + 
+                    #geom_point(aes(size=wi)) + 
                     opts(title=title,  legend.position="none") + 
                     xlim(-1.7, 1.7) + 
                     ylim(.028, .5) + 
@@ -1487,7 +1361,7 @@ FunnelPlot <- function(meta, method="random",  title=NULL) {
     funnel <- ggplot(meta,  aes(y = se.z.tau,  x = z))  +  
                     geom_vline(colour="black", linetype=1, 
                     xintercept=omnibus.z.tau) + 
-                    geom_point(aes(size=wi.tau)) + 
+                    #geom_point(aes(size=wi.tau)) + 
                     opts(title=title,  legend.position="none") + 
                     xlim(-1.7, 1.7) + 
                     ylim(.028, .5) + 
@@ -1740,7 +1614,7 @@ Wifun <-  function(meta) {
   meta$l.ci95 <- (exp(2*meta$l.ci95z)-1)/(exp(2*meta$l.ci95z) + 1)
   meta$u.ci95 <- (exp(2*meta$u.ci95z)-1)/(exp(2*meta$u.ci95z) + 1)  
   meta$z.score <- meta$z/sqrt(meta$var.z)
-  meta$p.value <- 2*(1-pt(abs(meta$z.score),  meta$n-1))
+  meta$p.value <- 2*pnorm(abs(meta$z.score), lower.tail=FALSE)
   meta$wi <-  1/meta$var.z  # computing weight for each study
   meta$wiTi <- meta$wi*meta$z  # used to calculate omnibus
   meta$wiTi2 <- meta$wi*(meta$z)^2  # used to calculate omnibus
